"Node: a class to represent a Node of a tree"

import myutils

class Node(object):
    def __init__(self,anc,bl=1,name=""):
        self.ancestor=anc                   # ancestor Node
        self.children=[]                    # children Nodes
        self.branch=bl                      # branch length of branch leading to actual node
        self.name=name                      # name of node, eg. taxa name
        self.depth=self.get_depth()         # depth is just number of nodes till root
        self.height=0                       # height is dependent on branch length till leaves!
        self.split=None                     # Split object belonging to node

    # depth of Node, root has depth 0
    def get_depth(self):
        return len(self.ancestors())-1 

    # compute heights for all nodes
    def insert_height(self):
        if len(self.children)==0:self.height=0
        else: self.height=myutils.mean([c.insert_height()+c.branch for c in self.children])
        return self.height

    # append child to list
    def set_child(self,child): self.children.append(child) 

    # return ancestor nodes and self node in order root->bottom
    def ancestors(self): 
        if self.ancestor==None: return [self.name]
        else: return self.ancestor.ancestors()+[self.name]

    # return self node + children traversing
    def traverse(self): 
        trav = [self]
        for child in self.children:
            trav+=child.traverse()
        return trav

    # get list of leaf names
    def get_leaves(self,names=True): 
        if len(self.children)==0:
            if names:return [self.name]
            else:return [self]
        l=[]
        for child in self.children:
            l+=child.get_leaves(names)
        self.leaves=l
        return l

    # create newick string from tree    
    def newick(self,intern=False,bl=False): 
        if len(self.children)==0:
            if bl: return "%s:%f" %(self.name,self.branch)
            else: return self.name
        s="("+self.children[0].newick(intern,bl)
        for i in range(1,len(self.children)):
            s+=","+self.children[i].newick(intern,bl)
        s+=')'
        if self.ancestor==None: return s
        if intern:s+=self.name
        if bl:s+=":%f" %self.branch
        return s

    # change node.split of every node
    def insert_split_rep_miss(self,all,root=0): 
        import Split
        
        all_leaves=self.get_leaves()
        occ=sum([1<<i for i in range(0,len(all)) if all[i] in all_leaves])
        if not root:root=all_leaves[0]
        
        self.get_split(all,root,occ) #insert splits recursivly but dont insert in root node
        self.split=Split.Split(0,node=self,occ=occ) #set empty split

    # insert and return splits recursivly    
    def get_split(self,all,root,occ): 
        import Split
        
        if self.split:return split.split
        if not self.children:
            if self.name!=root:split=1<<all.index(self.name)
            else:split=occ-(1<<all.index(self.name)) #all 1 except root
        else:
            split=self.children[0].get_split(all,root,occ)
            for i in range(1,len(self.children)):
                split^= self.children[i].get_split(all,root,occ)
        self.split=Split.Split(split,node=self,occ=occ)
        return split

    # return all splits
    def get_split_rep(self,term=False,empty=False):
        all_leaves=self.get_leaves()
        n=len(all_leaves)

        all_splits=[]
        for node in self.traverse():
            if not empty and node.split.split==0:continue
            if not term and not node.children:continue
            all_splits.append(node.split)
        return all_splits

