"""Split - a class to represent a split
           defined by two bitvectors: split (codes taxa on one site) and occ (codes all taxa present)
SplitSet - a class to represent a set of splits (usually the splits of a tree)
"""

import myutils

class Split:
    def __init__(self,splitstr,node=None,occ=0,weight=1): #set occ only if splitstr already int
        
        if type(splitstr)!=list: #already coded without missing data
            self.split=splitstr
            if not occ: self.occ=Split.max
            else:self.occ=occ
            
        else: #splitstr is list of 0,+,- need to code it binary
            self.split=sum([Split.infmap[splitstr[i]]<<i for i in range (0,len(splitstr))]) #binary coding information (missing positions also 0)
            self.occ=sum([Split.occmap[splitstr[i]]<<i for i in range (0,len(splitstr))])   #binary coding occurence (missing position 0, others 1)
        
        bc=myutils.bitcount(self.split)
        if bc==1 or bc==myutils.bitcount(self.occ)-1:self.term=True
        else:self.term=False

        self.score=0  # 1 - present in supertree, 0 - not present, list where entries represent supertree splits
        self.index=-1 # index in corresponding SplitSet
        self.weight=weight

    def set_weight(self,w): self.weight=w

    def set_index(self,i): self.index=i

    # string representation
    def __repr__(self):
        return "split %u occ %u term %u score %s index %u weight %1.4f" %(self.split,self.occ,self.term,str(self.score),self.index,self.weight)

    # return score 
    def get_score(self): #score=True if subsplit resp list over all supertree splits, then true if one true
        if type(self.score)==type([]):
            return int(max(self.score))*self.weight #for gene tree splits
        return self.score*self.weight

    # decode binary representation into string list again
    def get_string(self): 
        str=[]
        for i in range(0,len(SplitSet.taxa)):
            if (1<<i)&self.split:str.append("1")
            elif not ((1<<i)&self.occ):str.append("-")
            else: str.append("0")
        return str

    # return true if self is a subsplit of otherstr (i.e. it is equal without considering 0-occurences)
    def subsplit(self,otherstr): #otherstr already int binary coding
        return not bool((self.split ^ otherstr)&self.occ) or not bool((Split.max-self.split ^ otherstr)&self.occ)

        
class SplitSet:
    def __init__(self,tree,term=False,empty=False,reroot=True):

        if type(tree)==str: self.node=myutils.parse_newick(tree,unroot=True) #tree in newick format
        else: self.node=tree                                                 #tree already Node
        
        l=self.node.get_leaves()
        for i in range(len(SplitSet.taxa)-1,-1,-1): # find root, reverse order, such that largest taxa is root 
            t=SplitSet.taxa[i]
            if t in l:break
        if reroot: self.node=myutils.reroot(self.node,t,unroot=True)
        
        self.node.insert_split_rep_miss(SplitSet.taxa,root=t)      #insert node.split
        self.splits=self.node.get_split_rep(term=term,empty=empty) #get split representations
        
        i=0
        for s in self.splits:
            s.set_index(i)
            i+=1

    # methods such that a SplitSet can be used like a list (iterating over splits)
    def __len__(self):
        return len(self.splits)

    def __getitem__(self,i):
        return self.splits[i]
